#!/usr/bin/perl

# Copyright (c) Böcherer Angewandte Informatik (www.bai.de)
# example for a forward script to integrate SendXMS into a mail server (body (first 500 characters) will be sent to the recipient given in the subject header)
# generate a .forward file for the mail account to use which executes this script

use strict;

# maybe you have to install some additional (free) Perl modules
use Email::Simple;
use Email::MIME::ContentType;
use Text::Iconv;
use Encode;

my ($mail, $body, $subject, $cv, $ct, $wd);

$mail = Email::Simple->new(join("", <STDIN>));                    # read complete mail from stdin
$body = $mail->body;                                              # get message body (part to send

$subject = decode('MIME-Header', $mail->header("Subject"));       # get (and decode) subject


if ($subject ne "") {                                             # only if subject (recipient) is given)
  $ct = parse_content_type($mail->header("Content-Type"));        # get Content-Type (including charset)
  
  if (($ct->{attributes}->{discrete} eq "") || (($ct->{attributes}->{discrete} eq "text") && (($ct->{attributes}->{composite} eq "") || ($ct->{attributes}->{composite} eq "plain")))) {
                                                                  # accept only text/plain messages
    if ($mail->header("Content-Transfer-Encoding") eq "quoted-printable") {
                                                                  # decode quoted-printable
      $body =~ s/\=\n/\n/;
      $body =~ s/\=([a-fA-F0-9][a-fA-F0-9])/pack("C",hex($1))/eg;
    }

    if ($ct->{attributes}->{charset} ne "") {                     # convert body to UTF-8
      $ct->{attributes}->{charset} = lc ($ct->{attributes}->{charset});
      $cv = Text::Iconv->new($ct->{attributes}->{charset}, 'UTF-8');
      $body = $cv->convert($body);
    }

    $body =~ s/\t/\\\t/;                                          # replace TABs
    $body =~ s/\r/\\\r/;                                          # replace CRs
    $body =~ s/\"/\\\"/g;                                         # replace double quotes
    
    $body = substr($body, 0, 500);                                # truncate body to 500 characters

    `/usr/local/SendXMS/sendxms -XUTF8IO -N5 $subject -M\"$body\" > /dev/null`;  # execute SendXMS
  }
}
