#!/usr/bin/perl

# Copyright (c) Böcherer Angewandte Informatik (www.bai.de)
# example for a CGI script to integrate SendXMS into a WEB server

use strict;
use HTML::Entities;

my ($cmd, $i, %form, $field, @formField, @provider, @device, @phonebook, @groups, $chapter, $key, $value, %ini, $data, $name, $iniFile);

# open an given file in Windows Ini file format
sub OpenIniFile {
   my ($fp, $rc);

   $fp = $_[0];
   $rc = 0;
   open ($fp, "<".$_[1]) or ($rc = -1);

   if ($rc >= 0) {
      my $bom;

      read $fp, $bom, 3;

      my $o1 = ord (substr ($bom,0,1));
      my $o2 = ord (substr ($bom,1,1));
      my $o3 = ord (substr ($bom,2,1));

      if (($o1 != 0xEF) && ($o2 == 0xBB) || ($o3 != 0xBF)) { # it is not UTF-8
         seek ($fp, 0, 0);
      }
   }
   return ($rc);
}

# just close an given file
sub CloseIniFile {
   my ($fp);

   $fp = $_[0];
   close ($fp);
}

# read one line from an Ini file 
sub ReadIniFileLine {
   my ($fp, $zeile, $len, $rc, $i);

   $fp = $_[0];
   $value = "";
   $len = 0;
   while ((!eof ($fp)) && ($len == 0)) {
      $zeile = <$fp>;
      chomp ($zeile);
      $zeile =~ s/;.*$//g;
      $zeile =~ s/^\s+//g;
      $zeile =~ s/\s+$//g;
      $len = length ($zeile);
   }
   if ($len > 0) {
      if ((substr ($zeile, 0, 1) eq '[') && (substr ($zeile, -1)  eq ']')) {
# new chapter
         $chapter = substr ($zeile, 1, $len - 2);
         $key = "";
         $rc = 0;
      } else {
# key=value pair
         if (($i = index ($zeile, "=")) >= 0) {  #split line in key - value pairs
            $key = substr ($zeile, 0, $i);
            $value = substr ($zeile, $i + 1);
            $rc = 1;
         } else {
            $key = $zeile;
         }
      }
      $_[1] = $chapter;
      $_[2] = $key;
      $_[3] = $value;
   } else {
      $_[1] = "";
      $_[2] = "";
      $_[3] = "";
      $rc = -1;
   }
   $_[1] =~ s/^\s+//g;
   $_[1] =~ s/\s+$//g;
   $_[2] =~ s/^\s+//g;
   $_[2] =~ s/\s+$//g;
   $_[3] =~ s/^\s+//g;
   $_[3] =~ s/\s+.$//g;
   $_[1] = lc ($_[1]);
   return ($rc);
}

# print header for a HTML document
sub PrintHeader {
   my $page;

   $page = $_[0];
   print "Content-type: text/html\n\n";
   print "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n";
   if ($ini{'language'} eq "german") {
      print "<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"de\" lang=\"de\">\n";
   } else {
      print "<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\">\n";
   }
   print "<head>\n";
   print "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"/>\n";
   print "<title>SendXMS</title>\n";
   print "<script type=\"text/javascript\">\n";
   print "<!--\n/* <![CDATA[ */\n";
   print "function SetFocus() {\n";
   if ($page == 0) { # Formular
      print "   if (document.forms[\"SendXMS\"].elements[\"addrtype0\"].checked == true)\n";
      print "      document.forms[\"SendXMS\"].elements[\"phone\"].focus();\n";
      print "   else if ((document.forms[\"SendXMS\"].elements[\"addrtype1\"] != null) && (document.forms[\"SendXMS\"].elements[\"addrtype1\"].checked == true))\n";
      print "      document.forms[\"SendXMS\"].elements[\"phonebook\"].focus();\n";
      print "   else if ((document.forms[\"SendXMS\"].elements[\"addrtype2\"] != null) && (document.forms[\"SendXMS\"].elements[\"addrtype2\"].checked == true))\n";
      print "      document.forms[\"SendXMS\"].elements[\"group\"].focus();\n";
   } elsif ($page == 1) {  # SendXMS output
      print "   document.forms[\"SendXMS\"].elements[\"button\"].focus();\n";
   } elsif ($page == 2) {  # Error
      print "   document.forms[\"SendXMS\"].elements[\"button\"].focus();\n";
   }
   print "}\n";
   if ($page == 0) { # Formular
      print "\nfunction ChkValues() {\n";
      print "   var flag = Boolean;\n\n";
      print "   flag = false;\n";
      print "   if ((document.forms[\"SendXMS\"].elements[\"addrtype0\"].checked == true) && (document.forms[\"SendXMS\"].elements[\"phone\"].value.length > 0))\n";
      print "      flag = true;\n";
      print "   else if ((document.forms[\"SendXMS\"].elements[\"addrtype1\"] != null) && (document.forms[\"SendXMS\"].elements[\"addrtype1\"].checked == true) && (document.forms[\"SendXMS\"].elements[\"phonebook\"].value.length > 0))\n";
      print "      flag = true;\n";
      print "   else if ((document.forms[\"SendXMS\"].elements[\"addrtype2\"] != null) && (document.forms[\"SendXMS\"].elements[\"addrtype2\"].checked == true) && (document.forms[\"SendXMS\"].elements[\"group\"].value.length > 0))\n";
      print "      flag = true;\n";
      print "   if (flag == false)\n";
      if ($ini{'language'} eq "german") {
         print "      window.alert('Kein Empfänger angegeben!!');\n";
      } else {
         print "      window.alert('No recipient specified!!');\n";
      }
      print "   else {\n";
      print "      flag = false;\n\n";
      print "      if (document.forms[\"SendXMS\"].elements[\"sms\"].value.length > 0)\n";
      print "         flag = true;\n";
      print "      if (flag == false)\n";
      if ($ini{'language'} eq "german") {
         print "         window.alert('Keine Nachricht angegeben!!');\n";
      } else {
         print "         window.alert('No message specified!!');\n";
      }
      print "   }\n";
      print "   return (flag);\n";
      print "}\n";
   }
   print "/* ]]> */\n-->\n";
   print "</script>\n";
   print "</head>\n";
   print "<body onload=\"SetFocus()\">\n";
}

# print footer for a HTML document
sub PrintFooter {
   print "<p align=\"right\" style=\"font-size: 9pt;\">\n";
   print "&copy; BAI - Böcherer Angewandte Informatik\n";
   print "</p>\n";
   print "<hr/>\n";
   print "<table border=\"0\" width=\"100%\">\n";
   print "   <tr>\n";
   print "      <td valign=\"top\" align=\"left\">\n";
   print "         <address>\n";
   print "            <strong>BAI - Böcherer&nbsp;Angewandte&nbsp;Informatik</strong><br />\n";
   print "            Dipl.-Inform. Wolfgang Böcherer - CEO<br />\n";
   print "            Kaiserallee&nbsp;99<br />\n";
   print "            76185&nbsp;Karlsruhe, Germany<br />\n";
   print "         </address>\n";
   print "      </td>\n";
   print "      <td valign=\"top\" align=\"right\">\n";
   print "            Tel:&nbsp;+49&nbsp;721&nbsp;8304330<br />\n";
   print "            Fax:&nbsp;+49&nbsp;721&nbsp;8304333<br />\n";
   print "            Email:&nbsp;<a href=\"mailto:info\@bai.de\" onmouseover=\"window.status='mail us your comments'; return true;\">info\@bai.de</a><br />\n";
   print "            WWW:&nbsp;<a href=\"http://www.bai.de\" onmouseover=\"window.status='Homepage - Böcherer Angewandte Informatik'; return true;\">http://www.bai.de</a>\n";
   print "      </td>\n";
   print "   </tr>\n";
   print "</table>\n";
   print "</body>\n";
   print "</html>\n";
   exit (0);
}

# print Error page
sub Error {
   PrintHeader (2);
   print ("<h3>ERROR!!!</h3>".$_[0]."<br />\n");
   print "<script type=\"text/javascript\">\n";
   print "<!--\n/* <![CDATA[ */\n";
   print "document.write (\"<form id=\\\"SendXMS\\\">\\n<p align=\\\"center\\\">\\n\");\n";
   if ($ini{'language'} eq "german") {
      print ("document.write (\"<input id=\\\"button\\\" type=\\\"button\\\" value=\\\"Zur&uuml;ck\\\" onClick=\\\"history.back()\\\">\\n\");\n");
   } else {
      print ("document.write (\"<input id=\\\"button\\\" type=\\\"button\\\" value=\\\"Back\\\" onClick=\\\"history.back()\\\">\\\n\");\n");
   }
   print "document.write (\"<\\/form>\\n\");\n";
   print "/* ]]> */\n-->\n";
   print "</script>\n";
   PrintFooter ();
}

# check request methode
if ($ENV{'REQUEST_METHOD'} eq "POST") {
   read(STDIN, $data, $ENV{'CONTENT_LENGTH'});
} else {
   $data = $ENV{'QUERY_STRING'};
}
# read form data
@formField = split(/&/, $data);
foreach $field (@formField) {
   $field =~ s/\+/ /g;
   ($name, $value) = split(/=/, $field, 2);
   $name =~ s/%(..)/pack("c",hex($1))/ge;
   $value =~ s/%(..)/pack("c",hex($1))/ge;
   $form{$name} = $value;
}
$|=1;
# read sendxms.ini
if (OpenIniFile (*iniFile, "sendxms.ini") >= 0) {
   while (ReadIniFileLine (*iniFile, $chapter, $key, $value) >= 0) {
      if ($chapter eq "sendxms") {
         $key = lc ($key);
         if (($key ne "") && ($value ne "")) {$ini{$key} = $value;}
      } elsif ($chapter eq "groups") {
         if (($key ne "") && ($value ne "")) {push (@groups, {'alias' => $key, 'file' => $value});}
      }
   }
   $ini{'path'} = $ini{'sendxms'};
   if (($i = rindex ($ini{'path'}, "/", length ($ini{'path'}))) >= 0) {
      $ini{'path'} = substr ($ini{'path'}, 0, $i + 1);
   } else {
      if (($i = rindex ($ini{'path'}, "\\", length ($ini{'path'}))) >= 0) {
         $ini{'path'} = substr ($ini{'path'}, 0, $i + 1);
      }
   }
   CloseIniFile (*iniFile);
} else {
   if ($ini{'language'} eq "german") {Error ("Kein Zugriff auf Datei 'sendxms.ini'!");}
   else {Error ("can't open the file 'sendxms.ini'!");}
}


if (($ENV{'PATH_INFO'} eq "/send") || ($ENV{'PATH_INFO'} eq "$ENV{'SCRIPT_NAME'}/send")) {
# generate and execute SendXMS command
   $cmd = $ini{'sendxms'};
   $cmd .= " --Utf8IO";
   if ($ini{'config'} ne "") {$cmd .= " -c".$ini{'config'};}
   if ($ini{'provider'} ne "") {$cmd .= " -r".$ini{'provider'};}
   if ($ini{'phonebook'} ne "") {$cmd .= " -b".$ini{'phonebook'};}
   if ($ini{'userexit'} ne "") {$cmd .= " -u".$ini{'userexit'};}
   if ($form{'verbose'} eq "1") {$cmd .= " -v";}
   if ($form{'spool'} eq "spool") {
      $cmd .= " -s";
   } else {
      if ($form{'spool'} eq "send") {$cmd .= " -n";}
   }
   if ($form{'originator'} ne "") {$cmd .= " -o".$form{'originator'};}
   if (($form{'device'} ne "") && ($form{'device'} ne "(auto)")) {$cmd .= " -d".$form{'device'};}
   if ($form{'addrtype'} eq "phone") {
      if ($form{'phone'} eq "") {
         if ($ini{'language'} eq "german") {Error ("Kein Empfänger angegeben!!");}
         else {Error ("No recipient specified!!");}
      }
      if (($form{'provider'} ne "") && ($form{'provider'} ne "(auto)")) {$cmd .= " -p".$form{'provider'};}
      $cmd .= " ".$form{'phone'};
   } elsif ($form{'addrtype'} eq "phonebook") {
      if ($form{'phonebook'} eq "") {
         if ($ini{'language'} eq "german") {Error ("Kein Empfänger angegeben!!");}
         else {Error ("No recipient specified!!");}
      }
      ($key, $value) = split(/;/, $form{'phonebook'}, 2);
      $cmd .= " -p".$key;
      $cmd .= " ".$value;
   } elsif ($form{'addrtype'} eq "group") {
      if ($form{'group'} eq "") {
         if ($ini{'language'} eq "german") {Error ("Kein Empfänger angegeben!!");}
         else {Error ("No recipient specified!!");}
      }
      $cmd .= " -g".$form{'group'};
   }
   if ($form{'sms'} eq "") {
      if ($ini{'language'} eq "german") {Error ("Keine Nachricht angegeben!!");}
      else {Error ("No message specified!!");}
   }
   my $msg = $form{'sms'};
   $msg =~ s/\"/\\\"/g;
   $cmd .= " \"".$msg."\"";
   PrintHeader (1);
   if ($ini{'language'} eq "german") {
      print "<b><i>SendXMS</i></b> wird aufgerufen, bitte warten...<p />\n";
   } else {
      print "<b><i>SendXMS</i></b> will be executed, please wait...<p />\n";
   }
   if (!$ENV{'SERVER_PROTOCOL'}) {$ENV{'SERVER_PROTOCOL'} = "HTTP/1.0";}
   my $sendxmsOutput = `$cmd`;
   print "<pre>\n".encode_entities($sendxmsOutput, "<>&")."</pre>\n";
   print "<form id=\"SendXMS\" action=\"$ENV{'SCRIPT_NAME'}\">\n<p align=\"center\">\n";
   if ($ini{'language'} eq "german") {
      print "<input id=\"button\" type=\"submit\" value=\"Neue Nachricht\" />\n";
   } else {
      print "<input id=\"button\"  type=\"submit\" value=\"new message\" />\n";
   }
   print "</p>\n</form>\n"
} else {
# read .cfg file
   if ($ini{'config'} ne "") {
      if (OpenIniFile (*iniFile, $ini{'config'}) >= 0) {
         while (ReadIniFileLine (*iniFile, $chapter, $key, $value) >= 0) {
            $key = lc ($key);
            if (($chapter eq "device") && ($key eq "name")) {push (@device, $value);}
         }
         CloseIniFile (*iniFile);
      } else {
         if ($ini{'language'} eq "german") {Error ("Kein Zugriff auf Datei '".$ini{'config'}."'!");}
         else {Error ("can't open the file '".$ini{'config'}."'!");}
      }
   }

# read .pro file
   if ($ini{'provider'} ne "") {
      if (OpenIniFile (*iniFile, $ini{'provider'}) >= 0) {
         while (ReadIniFileLine (*iniFile, $chapter, $key, $value) >= 0) {
            $key = lc ($key);
            if (($chapter ne "") && ($key eq "") && ($value eq "")) {
               my $i = 0;

               while ($i <@provider) {
                  if ($provider[$i] eq $chapter) {$i = @provider + 1;}
                  $i++;
               }
               if ($i <= @provider) {push (@provider, $chapter);}
            }
         }
         CloseIniFile (*iniFile);
         if (@provider <= 0) {Error ("No provider configured!");}
      } else {
         if ($ini{'language'} eq "german") {Error ("Kein Zugriff auf Datei '".$ini{'provider'}."'!");}
         else {Error ("can't open the file '".$ini{'provider'}."'!");}
      }
   }

# read .pbk file
   if ($ini{'phonebook'} ne "") {
      if (OpenIniFile (*iniFile, $ini{'phonebook'}) >= 0) {
         while (ReadIniFileLine (*iniFile, $chapter, $key, $value) >= 0) {
            if (($chapter ne "") && ($chapter ne "*") && ($key ne "") && ($value ne "")) {push (@phonebook, {'alias' => $key, 'provider' => $chapter});}
         }
         CloseIniFile (*iniFile);
      } else {
         if ($ini{'language'} eq "german") {Error ("Kein Zugriff auf Datei '".$ini{'phonebook'}."'!");}
         else {Error ("can't open the file '".$ini{'phonebook'}."'!");}
      }
   }

# generate HTML form
   PrintHeader (0);

   print "<form id=\"SendXMS\" method=\"post\" action=\"$ENV{'SCRIPT_NAME'}/send\" onsubmit=\"return(ChkValues())\">\n";
   print "<table width=\"100%\" border=\"0\">\n";
   print "<tr><td>\n";
   if ($ini{'language'} eq "german") {
      print "<b>Absender:</b>\n";
   } else {
      print "<b>Originator:</b>\n";
   }
   print "<input type=\"text\" id=\"originator\" name=\"originator\" value=\"\"";
   if ($ENV{'REMOTE_USER'}) {
      print $ENV{'REMOTE_USER'};
      if ($ENV{'REMOTE_HOST'}) {print "@".$ENV{'REMOTE_HOST'};}
   }
   print "\" size=\"30\" />\n";
   print "\n";
   print "</td><td align=\"right\">\n";
   print "<span style=\"font-size: 14pt;\"><b><i>SendXMS</i></b></span><sup>&reg;</sup>\n";
   print "</td></tr>\n";
   print "</table><br />\n";
   if ($ini{'language'} eq "german") {
      print "<b>Empfänger:</b><br />\n";
   } else {
      print "<b>Receiver:</b><br />\n";
   }
   print "<table border=\"1\" width=\"100%\" style=\"border-collapse: collapse;\">\n";
   print "<tr><td>\n";
   print "<input type=\"radio\" checked=\"checked\" name=\"addrtype\" id=\"addrtype0\" value=\"phone\" />";
   if ($ini{'language'} eq "german") {
      print " Telefonnummer\n";
   } else {
      print " phone number\n";
   }
   print "</td><td>\n";
   print "<input type=\"text\" id=\"phone\" name=\"phone\" value=\"\" size=\"30\" />";
   if ($ini{'language'} eq "german") {
      print " Telefonnummer<br />\n";
   } else {
      print " phone number<br />\n";
   }
   print "<select id=\"provider\" name=\"provider\" size=\"1\">\n";
   print "<option selected=\"selected\">(auto)</option>\n";
   for ($i = 0; $i < @provider; $i++) {
      print "<option>".$provider[$i]."</option>\n";
   }
   if ($ini{'language'} eq "german") {
      print "</select> Provider\n";
   } else {
      print "</select> provider\n";
   }
   print "</td></tr>\n";
   if (@phonebook > 0) {
      print "<tr><td>\n";
      print "<input type=\"radio\" name=\"addrtype\" id=\"addrtype1\" value=\"phonebook\" />";
      if ($ini{'language'} eq "german") {
         print " Telefonbuch\n";
      } else {
         print " phonebook\n";
      }
      print "</td><td>\n";
      print "<select id=\"phonebook\" name=\"phonebook\" size=\"1\">\n";
      for ($i = 0; $i < @phonebook; $i++) {
         print "<option value=\"".$phonebook[$i]->{'provider'}.";".$phonebook[$i]->{'alias'}."\">".$phonebook[$i]->{'alias'}." [".$phonebook[$i]->{'provider'}."]</option>\n";
      }
      print "</select>\n";
      print "</td></tr>\n";
   }
   if (@groups > 0) {
      print "<tr><td>\n";
      print "<input type=\"radio\" name=\"addrtype\" id=\"addrtype2\" value=\"group\" />";
      if ($ini{'language'} eq "german") {
         print " Gruppe\n";
      } else {
         print " group\n";
      }
      print "</td><td>\n";
      print "<select id=\"group\" name=\"group\" size=\"1\">\n";
      for ($i = 0; $i < @groups; $i++) {
         print "<option value=\"".$groups[$i]->{'file'}."\">".$groups[$i]->{'alias'}."</option>\n";
      }
      print "</select>\n";
      print "</td></tr>\n";
   }
   print "</table>\n";
   print "<br />\n";
   if ($ini{'language'} eq "german") {
      print "<b>Nachricht:</b><br />\n";
   } else {
      print "<b>Message:</b><br />\n";
   }
   print "<table border=\"1\" width=\"100%\" style=\"border-collapse: collapse;\">\n";
   print "<tr>\n";
   print "<td>\n";
   print "<textarea id=\"sms\" name=\"sms\" rows=\"3\" cols=\"60\"></textarea>\n";
   print "</td></tr>\n";
   print "</table>\n";
   print "<br />\n";
   if ($ini{'language'} eq "german") {
      print "<b>Optionen:</b> (für Experten)<br />\n";
   } else {
      print "<b>Options: (for experts)</b><br />\n";
   }
   print "<table width=\"100%\" border=\"0\">\n";
   print "<tr><td>\n";
   print "<input type=\"checkbox\" id=\"verbose\" name=\"verbose\" value=\"1\" />verbose -- ";
   if ($ini{'language'} eq "german") {
      print "<select id=\"spool\" name=\"spool\" size=\"1\" title=\"Nachricht spoolen oder direkt versenden\">\n";
   } else {
      print "<select id=\"spool\" name=\"spool\" size=\"1\" title=\"send or spool the message\">\n";
   }
   print "<option selected=\"selected\" value=\"(auto)\" >(auto)</option>\n";
   if ($ini{'language'} eq "german") {
      print "<option value=\"send\">senden</option>\n";
      print "<option value=\"spool\">spoolen</option>\n";
   } else {
      print "<option value=\"send\">send</option>\n";
      print "<option value=\"spool\">spool</option>\n";
   }
   print "</select>\n";
   if (@device > 1) {
      if ($ini{'language'} eq "german") {
         print " -- <select id=\"device\" name=\"device\" title=\"Gerät auswählen\"size=\"1\">\n";
      } else {
         print " -- <select id=\"device\" name=\"device\" title=\"select a device to use\"size=\"1\">\n";
      }
      print "<option selected=\"selected\">(auto)</option>\n";
      for ($i = 0; $i < @device; $i++) {
         print "<option>".$device[$i]."</option>\n";
      }
   }
   print "</td><td align=\"right\">\n";
   if ($ini{'language'} eq "german") {
      print "<input type=\"submit\" id=\"Send\" value=\"Ausführen\" />\n";
      print "<input type=\"reset\" value=\"Löschen\" />\n";
   } else {
      print "<input type=\"submit\" id=\"Send\" value=\"Execute\" />\n";
      print "<input type=\"reset\" value=\"Reset\" />\n";
   }
   print "</td></tr>\n";
   print "</table>\n";
   print "</form>\n";
}
PrintFooter ();
