<?php

################################################################
# Copyright (c) Böcherer Angewandte Informatik (www.bai.de)    #
# just a simple sample for generating spool files              #
################################################################

$spoolDir = "./spool/";
date_default_timezone_set('Europe/Berlin');

function xmsInitialize($argv) {
   $spool = [];
   $spool['Chapter'] = '';
   $spool['StatusReportRequest'] = FALSE;
   $spool['LocalId'] = '';
   $spool['OriginatingAddress'] = '';
   $spool['SplitCount'] = 0;

   $options = getopt('FN:o:O:p:', [], $optind);
   if (array_key_exists('F' , $options)) $spool['StatusReportRequest'] = TRUE;
   if (!empty($options['N'])) $spool['SplitCount'] = intval($options['N']);
   if (!empty($options['o'])) $spool['LocalId'] = $options['o'];
   if (!empty($options['O'])) $spool['OriginatingAddress'] = $options['O'];
   if (!empty($options['p'])) $spool['Chapter'] = $options['p'];
   $pos_args = array_slice($argv, $optind);
   if ((!empty($pos_args[0])) && (!empty($pos_args[1]))) {
      $spool['Phone'] = $pos_args[0];
      $spool['XMS'] = xmsEscapeMsgText($pos_args[1]);
   } else {
      xmsExit("invalide command line syntax\n");
   }
   return($spool);
}

function xmsExit($msg) {
   echo "\n$msg\n";
   exit(0);
}

function xmsEscapeMsgText($xms) {
   $xms = preg_replace('/\\\/', '\\\\\\', $xms);
   $xms = preg_replace('/\n/', '\\n', $xms);
   $xms = preg_replace('/\r/', '\\r', $xms);
   $xms = preg_replace('/\t/', '\\t', $xms);
   return($xms);
}

function xmsCreateSpoolFile($spool) {
   global $spoolDir;

   if (!is_dir($spoolDir."/".$spool['Chapter'])) mkdir ($spoolDir."/".$spool['Chapter'], 0777, TRUE);

   $attempt = 0;
   while ($attempt < 3) {
      try {
         list($usec, $sec) = explode(" ", microtime());
         $prefix = "xms";

         if (PHP_INT_SIZE <= 4) {
            $prefix = sprintf("xms%08lx%05lx%08lx%08lx", $sec, $usec, getmypid(), 1);
         } else {
            $prefix = sprintf("xms%016lx%05lx%016lx%016lx", $sec, $usec, getmypid(), 1);
         }
         if ($fh = fopen(tempnam($spoolDir."/".$spool{'Chapter'}."/", $prefix), 'w')) {
            if (flock($fh, LOCK_NB|LOCK_EX)) {
               $attempt = 99;
            } else {
               fclose($fh);
               $attempt++;
            }
         } else {
            $attempt++;
         }
      } catch (Exception $e) {
         $attempt++;
         if ($attempt >= 3) xmsExit("can't spool message\n");
      }
   }

   if ($fh===false) {
      xmsExit("can't spool message\n");
   }

   try {
      fwrite($fh, "; encoding=UTF-8\n[".$spool{'Chapter'}."]\n");
      fwrite($fh, "Created=".strftime("%Y%m%d%H%M%S", $sec)."\n");
      fwrite($fh, "Count=0\n");
      if ($spool['StatusReportRequest']) fwrite($fh, "StatusReportRequest=TRUE\n");
      if ($spool['LocalId']) fwrite($fh, "LocalId=".$spool['LocalId']."\n");
      if ($spool['OriginatingAddress']) fwrite($fh, "OriginatingAddress=".$spool['OriginatingAddress']."\n");
      if ($spool['SplitCount']) fwrite($fh, "Split=".$spool['SplitCount']."\n");
      fwrite($fh, "Phone=".$spool{'Phone'}."\n");
      fwrite($fh, "XMS=".$spool{'XMS'}."\n");
      fclose ($fh);
   } catch (Exception $e) {
      xmsExit("can't spool message\n");
   }
}

$spool=xmsInitialize($argv);
xmsCreateSpoolFile($spool);
exit(1);
?>
