#!/usr/local/bin/perl -w

################################################################
# Copyright (c) Böcherer Angewandte Informatik (www.bai.de)    #
# just a simple sample for generating spool files              #
################################################################

use strict;
use Config;
use threads;
use File::Temp;
use POSIX qw(strftime);
use Getopt::Long qw(:config gnu_getopt);
use Fcntl qw(:flock LOCK_EX LOCK_NB);
use Time::HiRes qw(gettimeofday);

my $spoolDir = "./spool/";
my %spool;

sub xmsExit {
   my $msg = shift;

   print $msg;
   exit(0);
}

sub xmsEscapeMsgText {
   my $xms = shift;

   $xms =~ s/\\/\\\\/g;
   $xms =~ s/\n/\\n/g;
   $xms =~ s/\r/\\r/g;
   $xms =~ s/\t/\\t/g;
   return($xms);
}

sub xmsInitialize {
   $spool{"Chapter"} = '';
   $spool{"StatusReportRequest"} = 0;
   $spool{"LocalId"} = '';
   $spool{"OriginatingAddress"} = '';
   $spool{"SplitCount"} = 0;

   local $SIG{__WARN__} = sub {xmsExit($_[0]);};
   GetOptions('-F' => \$spool{'StatusReportRequest'}, '-N=i' => \$spool{'SplitCount'}, '-o=s' => \$spool{'LocalId'}, '-O=s' => \$spool{'OriginatingAddress'}, '-p=s' => \$spool{'Chapter'});

   if (($ARGV[0]) && ($ARGV[1])) {
      $spool{"Phone"} = $ARGV[0];
      $spool{"XMS"} = xmsEscapeMsgText($ARGV[1]);
   } else {
      xmsExit("invalide command line syntax\n");
   }
}

sub xmsCreateSpoolFile {
   if ($spool{"Chapter"}) {
      my ($attempt, $fh, $prefix, $sec, $usec);

      mkdir $spoolDir;
      mkdir "$spoolDir/$spool{'Chapter'}";

      $attempt = 0;
      while ($attempt < 3) {
         eval {
            ($sec, $usec) = gettimeofday;
            $prefix = "xms";

            if ($Config{longsize} <= 4) {
               $prefix = sprintf("xms%08lx%05lx%08lx%08lx", $sec, $usec, $$, threads->tid());
            } else {
               $prefix = sprintf("xms%016lx%05lx%016lx%016lx", $sec, $usec, $$, threads->tid());
            }
            open($fh, ">", File::Temp::tempnam("$spoolDir/$spool{'Chapter'}/", $prefix)) or die "$!";
            flock($fh, LOCK_NB|LOCK_EX) or die "$!";
         };
         if ($@) {
            $attempt++;
            close($fh);
            xmsExit("can't spool message\n") if ($attempt >= 3);
         } else {
            $attempt = 99;
         }
      }

      eval {
         print($fh "; encoding=UTF-8\n[$spool{'Chapter'}]\n");
         print($fh "Created=", strftime("%Y%m%d%H%M%S", localtime), "\n");
         print($fh "Count=0\n");
         if ($spool{'StatusReportRequest'}) {print($fh "StatusReportRequest=TRUE\n")}
         if ($spool{'LocalId'}) {print($fh "LocalId=$spool{'LocalId'}\n")}
         if ($spool{'OriginatingAddress'}) {print($fh "OriginatingAddress=$spool{'OriginatingAddress'}\n")}
         if ($spool{'SplitCount'}) {print($fh "Split=$spool{'SplitCount'}\n")}
         print($fh "Phone=$spool{'Phone'}\n");
         print($fh "XMS=$spool{'XMS'}\n");
         close($fh);
      };
      if ($@) {xmsExit("can't spool message\n");}
   } else {
      xmsExit("you have to specify a provider name (-p)\n");
   }
}




xmsInitialize();
xmsCreateSpoolFile();
exit(1);
