/* Copyright (c) Böcherer Angewandte Informatik (www.bai.de) */
#define XMS_SPOOL_API_VERSION 1

#if defined (_WIN32 ) || defined (_WIN64)
#define XMSSPOOL_CALLCONV __stdcall __declspec(dllexport)
#else
#define XMSSPOOL_CALLCONV
#endif

#define XMS_SPOOLDIR 0     /* messages to send */
#define XMS_RECEIVEDIR 1   /* received messages */
#define XMS_UNSENTDIR 2    /* messages which couldn't be sent */
#define XMS_SENTDIR 3      /* already sent messages */

/* --------------------------------------------------
- These functions have to be implemented within a Shared Object or a DLL.
- These functions have to be thread safe.
- If one or more of these functions could not be loaded SendXMS will use the default file system API.
- All functions should return 0 if everything is OK.
- A messages is a string which conforms to a standard spool file format.
- The parameter provider is the name of a provider definition in the used .pro file
- The parameter key conforms to the file name of a standard spool file format. If a function returns a key maxKeyLen has to be respected (including the NULL character).
- All strings have to be UTF-8 encoded.
- In the initialisation function a pointer to any user defined structure can be returned. This pointer will be given
  to all other functions as the first parameter (to avoid global variables). Additionally the version of the implemented
  API can be returned (until now only version 1 is available).
- Usually the second parameter is an integer which specifies the directory of the given message (SPOOLDIR, RECEIVEDIR, UNSENTDIR oder SENTDIR).
  Whether the external storage structure uses in fact different directories (tables) or not doesn't matter.
- Records have to be locked exlusively (per thread)(use row locks not table locks).
-------------------------------------------------- */

/* initialise the API; returns a pointer to any Userdata and the version of the implemented API */
int XMSSPOOL_CALLCONV xmsSpoolApiInit (void **userdata, int *version);

/* release locks, memory, ... */
int XMSSPOOL_CALLCONV xmsSpoolApiExit (void *userdata);


/* generate a list with keys of messages in the given dir; return -1 if the list is empty */
int XMSSPOOL_CALLCONV xmsSpoolApiOpenMsgList (void *userdata, int dir, char *provider);

/* lock the next messages, allocate memory (sets *xmsLen to the size of the message) and return it; return -1 if no more messages are available  */
int XMSSPOOL_CALLCONV xmsSpoolApiGetNextMsg (void *userdata, int dir, char *provider, char *key, int maxKeyLen, char **xms, int *xmsLen);

/* free the memory allocated in xmsSpoolApiGetNextMsg (but do not release the lock!)*/
int XMSSPOOL_CALLCONV xmsSpoolApiFreeMsg (char *xms);

/* free the previously generated list */
int XMSSPOOL_CALLCONV xmsSpoolApiCloseMsgList (void *userdata, int dir, char *provider);


/* unlock the given record */
int XMSSPOOL_CALLCONV xmsSpoolApiUnlock (void *userdata, int dir, char *provider, char *key);

/* insert a new record and returns the generated primary key */
int XMSSPOOL_CALLCONV xmsSpoolApiInsert (void *userdata, int dir, char *provider, char *key, int maxKeyLen, char *xms, int xmsLen);

/* update the given record */
int XMSSPOOL_CALLCONV xmsSpoolApiUpdate (void *userdata, int dir, char *provider, char *key, char *xms, int xmsLen);

/* read a special record */
int XMSSPOOL_CALLCONV xmsSpoolApiRead (void *userdata, int dir, char *provider, int priority, char *msgId, char *key, char **xms, int *xmsLen);

/* delete the given record */
int XMSSPOOL_CALLCONV xmsSpoolApiDelete (void *userdata, int dir, char *provider, char *key);
