/* Copyright (c) Böcherer Angewandte Informatik (www.bai.de)                              */
/* example for an userexit implementation (either as a shared object or as an executable) */
/* do not make any output to stdout, use stderr instead                                   */

/* compile with 
all: userexit.c
	gcc -s -O1 -I. -o userexit userexit.c xmshash.c
	gcc -s -O1 -DSharedObject -D_REENTRANT -D_THREAD_SAFE -shared -fPIC -nostdlib -I. -o userexit.so userexit.c xmshash.c

or for Borland compiler
all: userexit.c
	bcc32 -v- -k- -tWC -tWM -O2 -w- -C -fp -lC -I. userexit.c xmshash.c -ouserexit.exe
	bcc32 -DSharedObject -v- -k- -tWD -tWM -O2 -w- -C -fp -I. userexit.c xmshash.c -ouserexit.dll

*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "xmshash.h"

#if defined (_WIN32) || defined (_WIN64)
#define XMSUSEREXIT_CALLCONV __stdcall __declspec(dllexport)
#else
#define XMSUSEREXIT_CALLCONV
#endif


int XMSUSEREXIT_CALLCONV UserExit (int cause, const char *xms, int xmsLen, char *changedXms) {
   int rc = -1;
   void *spool;

   *changedXms = 0;
   switch (cause) {
   case 1:
   case 2:
   case 3:
   case 4:
   case 8:
   case -1:
      if ((spool = xmsHashAlloc ((char*)xms, xmsLen)) != NULL) {
         rc = 0;
         switch (cause) {
         case 1:  /* message has been submitted */
            fprintf (stderr, "message %s sent: %s, %s, %s\n", xmsHashGet (spool, "msgid"), xmsHashGet (spool, "originatingaddress"), xmsHashGet (spool, "phone"), xmsHashGet (spool, "xms"));
            break;
         case 2:  /* message has been received */
            fprintf (stderr, "message %s received: %s, %s, %s\n", xmsHashGet (spool, "msgid"), xmsHashGet (spool, "originatingaddress"), xmsHashGet (spool, "phone"), xmsHashGet (spool, "xms"));
            break;
         case 3:  /* status report or deliver notification has been received */
            fprintf (stderr, "status/notification for %s received: %s, %s, %s\n", xmsHashGet (spool, "msgid"), xmsHashGet (spool, "originatingaddress"), xmsHashGet (spool, "phone"), xmsHashGet (spool, "xms"));
            break;
         case 4:  /* delete requeset has been submitted */
            fprintf (stderr, "message %s deleted: %s, %s, %s\n", xmsHashGet (spool, "msgid"), xmsHashGet (spool, "originatingaddress"), xmsHashGet (spool, "phone"), xmsHashGet (spool, "xms"));
            break;
         case 8:  /* MMS-Notification has been received (a MMS should be fetched) */
            {
               char cmd[500];

               sprintf (cmd, "./sendxms -s -aReceive -d\"%s\" -o\"%s\" -p\"%s\" \"%s\"", xmsHashGet (spool, "useddevice"), xmsHashGet (spool, "transactionid"), xmsHashGet (spool, "mmsprovidertouse"), xmsHashGet (spool, "addinfo"));
               system (cmd);
               fprintf (stderr, "fetch request for %s spooled\n", xmsHashGet (spool, "addinfo"));
            }
            break;
         case -1: /* any error has been occured */
            fprintf (stderr, "message marked as unsent: %s, %s, %s\n", xmsHashGet (spool, "originatingaddress"), xmsHashGet (spool, "phone"), xmsHashGet (spool, "xms"));
            break;
         default:
            break;
         }
         spool = xmsHashFree (spool);
      }
      break;
   default:
      rc = 0;
      break;
   }
   return (rc);
}

#ifndef SharedObject
int main (int argc, char **argv) {
   int rc = -1, cause, xmsLen;
   char *xms = NULL, changedXms[1000];

   if (argc > 1) {
      cause = atoi (argv[1]);
/* read spooled message from stdin (= a pipe) */
      xmsLen = 1; /* respect the terminating Null character */
      do {
         if ((rc = fread (changedXms, 1, sizeof (changedXms), stdin)) > 0) {
            if ((xms = realloc (xms, xmsLen + rc)) != NULL)
               memcpy (xms + xmsLen - 1, changedXms, rc);
            else
               return (-1);
            xmsLen += rc;
         }
      } while (rc == sizeof (changedXms));
      xmsLen--;   /* the terminating Null character is not part of the message */
      if (xms == NULL)
         return (-1);
      *(xms + xmsLen) = 0;
      memset (changedXms, 0, sizeof (changedXms));
/* execute the Userexit function (as it will be done in case of a shared object) */
      rc = UserExit (cause, xms, xmsLen, changedXms);
   }
   return (rc);
}
#endif
