#!/usr/bin/perl

################################################################
# Copyright (c) Böcherer Angewandte Informatik (www.bai.de)    #
# Example for an userexit with UserexitVersion=5               #
# Do not make any output to STDOUT, use STDERR instead         #
################################################################

my ($cause, %spool);

################################################################
# escape message text (replace \ with \\, CR with \n, ...      #
################################################################
sub EscapeMsgText {
   my $xms = shift;

   $xms =~ s/\\/\\\\/g;
   $xms =~ s/\n/\\n/g;
   $xms =~ s/\r/\\r/g;
   $xms =~ s/\t/\\t/g;
   return ($xms);
}

################################################################
# unescape message text (replace \\ with \, \n with CR, ...    #
################################################################
sub UnEscapeMsgText {
   my $xms = shift;

   $xms =~ s/(((\\\\)+)|([^\\])|^)(\\n)/$1\n/g;
   $xms =~ s/(((\\\\)+)|([^\\])|^)(\\r)/$1\r/g;
   $xms =~ s/(((\\\\)+)|([^\\])|^)(\\t)/$1\t/g;
   $xms =~ s/\\\\/\\/g;
   return ($xms);
}

################################################################
# read the spool file from STDIN (= a pipe)                    #
################################################################
sub ReadSpoolFile {
   my ($key, $value, $i);

   while (<STDIN>) {
      $_ =~ s/^\s+//g;
      $_ =~ s/\s+$//g;
      $_ =~ s/^;(.*)$//s;  # if line starts with a ';' it is a comment
      if ($spool{"chapter"} eq "") {  # at first there must be a provider (chapter)
         if ($_ =~ s/^\[(.*)\]/$1/s) {
            $spool{"chapter"} = $_;
         }
      } else {
         if (($i = index ($_, "=")) >= 0) {  # split line in key - value pairs
            $key = substr ($_, 0, $i);
            $value = substr ($_, $i + 1);
         } else {
            $key = $_;
            $value = "";
         }
         $key =~ s/^\s+//g;   # remove leading whitespaces
         $key =~ s/\s+$//g;   # remove trailing whitespaces
         $key = lc ($key);    # convert key name to lowercase
         $spool{$key} = $value;
      }
   }
}


$cause = $ARGV[0];
if ($cause == 1) {   # message has been submitted
   ReadSpoolFile ();
   printf STDERR "message %s sent: %s, %s, %s\n", $spool{msgid}, $spool{originatingaddress}, $spool{phone}, $spool{xms};
} elsif ($cause == 2) { # message has been received
   ReadSpoolFile ();
   printf STDERR "message %s received: %s, %s, %s\n", $spool{msgid}, $spool{originatingaddress}, $spool{phone}, $spool{xms};
} elsif ($cause == 3) { # status report or deliver notification has been received
   ReadSpoolFile ();
   printf STDERR "status/notification for %s received: %s, %s, %s\n", $spool{msgid}, $spool{originatingaddress}, $spool{phone}, $spool{xms};
} elsif ($cause == 4) { # delete request has been submitted
   ReadSpoolFile ();
   printf STDERR "message %s deleted: %s, %s, %s\n", $spool{msgid}, $spool{originatingaddress}, $spool{phone}, $spool{xms};
} elsif ($cause == 8) { # MMS-Notification has been received (a MMS should be fetched)
   ReadSpoolFile ();
   `./sendxms -s -aReceive -d"$spool{useddevice}" -o"$spool{transactionid}" -p"$spool{mmsprovidertouse}" "$spool{addinfo}"`;
   printf STDERR "fetch request for %s spooled\n", $spool{addinfo};
} elsif ($cause == -1) {   # any error has been occured
   ReadSpoolFile ();
   printf STDERR "message marked as unsent: %s, %s, %s\n", $spool{originatingaddress}, $spool{phone}, $spool{xms};
}

close STDIN;
close STDOUT;
exit (0);
