#!/usr/bin/python
# -*- coding: utf-8 -*-

################################################################
# Copyright (c) Böcherer Angewandte Informatik (www.bai.de)    #
# Example for an userexit with UserexitVersion=5               #
# Do not make any output to STDOUT, use STDERR instead         #
################################################################

import sys, string, re, os

spool = {}

def GetSpoolValue (keyname):
   if (keyname in spool):
      return (spool[keyname])
   return ("")

################################################################
# escape message text (replace \ with \\, CR with \n, ...      #
################################################################
def EscapeMsgText (xms):
   xms = re.sub ("\\\\", "\\\\\\\\", xms)
   xms = re.sub ("\n", "\\\\n", xms)
   xms = re.sub ("\r", "\\\\r", xms)
   xms = re.sub ("\t", "\\\\t", xms)
   return (xms)

################################################################
# unescape message text (replace \\ with \, \n with CR, ...    #
################################################################
def UnEscapeMsgText (xms):
   xms = re.sub ("(((\\\\\\\\)+)|([^\\\\])|^)(\\\\n)", "\\1\n", xms)
   xms = re.sub ("(((\\\\\\\\)+)|([^\\\\])|^)(\\\\r)", "\\1\r", xms)
   xms = re.sub ("(((\\\\\\\\)+)|([^\\\\])|^)(\\\\t)", "\\1\t", xms)
   xms = re.sub ("\\\\\\\\", "\\\\", xms)
   return (xms)

################################################################
# read the spool file from STDIN (= a pipe)                    #
################################################################
def ReadSpoolFile ():
   #Read the SendXMS Spool File from stdin (= a pipe)
   lines = sys.stdin.readlines ()
   for line in lines:
      line = line.strip ()                   #strip leading and trailing whitespace
      line = re.sub ("^;(.*)$", "", line)    #if line starts with a ';' it is a comment
      if (GetSpoolValue ("chapter") == ""):  #at first there must be a provider (chapter)
         chapter = re.sub ("^\[(.*)\]", "\\1", line)
         if (chapter != ""):
            spool["chapter"] = chapter
      else:                                  #store key/value pair in dictionary object
         i = line.find ("=")
         if (i >= 0):
            spool[line[:i].lower ().strip ()] = line[i + 1:]
         else:
            spool[line.strip ()] = ""
   return (spool)

def main():
   try:
      cause = int (sys.argv[1])
      if (cause == 1): # Message has been submitted
         ReadSpoolFile ()
         sys.stderr.write ("message " + GetSpoolValue("msgid") + " sent: " + GetSpoolValue("originatingaddress") + ", " + GetSpoolValue("phone") + ", " + GetSpoolValue("xms") + "\n")
      elif (cause == 2):  # Message has been received
         ReadSpoolFile ()
         sys.stderr.write ("message " + GetSpoolValue("msgid") + " received: " + GetSpoolValue("originatingaddress") + ", " + GetSpoolValue("phone") + ", " + GetSpoolValue("xms") + "\n")
      elif (cause == 3): # Status report or delivery notification has been received
         ReadSpoolFile ()
         sys.stderr.write ("status/notification for " + GetSpoolValue("msgid") + " received: " + GetSpoolValue("originatingaddress") + ", " + GetSpoolValue("phone") + ", " + GetSpoolValue("xms") + "\n")
      elif (cause == 4): # Delete request has been submitted
         ReadSpoolFile ()
         sys.stderr.write ("message " + GetSpoolValue("msgid") + " deleted: " + GetSpoolValue("originatingaddress") + ", " + GetSpoolValue("phone") + ", " + GetSpoolValue("xms") + "\n")
      elif (cause == 8): # MMS-Notification has been received (a MMS should be fetched)
         ReadSpoolFile ()
         os.system("./sendxms -s -aReceive -d\"" + GetSpoolValue("useddevice") + "\" -o\"" + GetSpoolValue("transactionid") + "\" -p\"" + GetSpoolValue("mmsprovidertouse") + "\" \"" + GetSpoolValue("addinfo") + "\"");
         sys.stderr.write ("fetch request for " + GetSpoolValue("addinfo") + " spooled\n")
      elif (cause < 0): # an error has occurred
         ReadSpoolFile ()
         sys.stderr.write ("message marked as unsent: " + GetSpoolValue("originatingaddress") + ", " + GetSpoolValue("phone") + ", " + GetSpoolValue("xms") + "\n")
      sys.stdin.close ()
   except:
      return(-1)
   return (0)

if __name__ == "__main__":
   main ()
