#!/usr/bin/perl

################################################################
# Copyright (c) Böcherer Angewandte Informatik (www.bai.de)    #
# Example for a Perls script to read a (MMS) spool file        #
# and to (m)unpack all (MIME) attachements                     #
################################################################

use MIME::Parser;

my $parser = new MIME::Parser;
my $count = 1;
my ($entity, $spool);

sub ReadSpoolFile {
   my ($key, $value, $i);
   my $spoolFile = shift;

   while (<$spoolFile>) {
      $_ =~ s/^\s+//g;
      $_ =~ s/\s+$//g;
      $_ =~ s/^;(.*)$//s;  # if line starts with a ';' it is a comment
      if ($spool{"chapter"} eq "") {  # at first there must be a provider (chapter)
         if ($_ =~ s/^\[(.*)\]/$1/s) {
            $spool{"chapter"} = $_;
         }
      } else {
         if (($i = index ($_, "=")) >= 0) {  #split line in key - value pairs
            $key = substr ($_, 0, $i);
            $value = substr ($_, $i + 1);
         } else {
            $key = $_;
            $value = "";
         }
         $key =~ s/^\s+//g;   # remove leading whitespaces
         $key =~ s/\s+$//g;   # remove trailing whitespaces
         $key = lc ($key);    # convert key name to lowercase
         $spool{$key} .= "\0" if (defined($spool{$key}));
         $spool{$key} .= $value;
      }
   }
}

sub dump_entity {
  my $entity = shift;
  my @parts = $entity->parts;
  my ($if, $of, $fn, $i);

  if (@parts) {
    map { dump_entity($_) } @parts;
  } else {
    $fn = scalar($entity->head->recommended_filename);
    if ($fn eq "") {
      $fn = $entity->bodyhandle->path;
      if ($fn eq "") {
        $fn = "sendxms.$$.part$count";
      }
    }
#    if (-e $fn) {
#      $i = 1;
#      while (-e "$fn.$i") {
#        $i++;
#      }
#      $fn = "$fn.$i";
#    }
    if ($if = $entity->open("r")) {
      if (open (OUT, ">",$fn)) {
        print(OUT <$if>);
        close (OUT);
#        print "file ", $fn, " (", scalar($entity->head->mime_type), ") saved\n";
      }
      $if->close;
    }
    $count++;
  }
}

# read complete spool file from STDIN or from given file
# if you want to use this in an userexit you will always receive the spool file over stdin (= PIPE)
if ($ARGV[0] eq "") {
  print "Reading from stdin:\n\n";
  ReadSpoolFile (\*STDIN);
} else {
  open (SPOOLFILE, "<$ARGV[0]");
  ReadSpoolFile (\*SPOOLFILE);
  close (SPOOLFILE);
}

# replace special characters
$spool{xms} =~ s/\\n/\n/g;
$spool{xms} =~ s/\\r/\r/g;
$spool{xms} =~ s/\\t/\t/g;
$spool{xms} =~ s/\\\\/\\/g;

$parser->output_to_core(false);
$entity = $parser->parse_data($spool{xms});
dump_entity($entity);
$entity->purge();
